@@ -16,7 +16,7 @@ use Attribute::Handlers;
 # Version of this module.
 #
 
-our $VERSION = '2.00';
+our $VERSION = '2.01';
 
 ### PACKAGE GLOBAL VARIABLE ###
 #
@@ -469,12 +469,25 @@ development times.
 From Perl module developer perspective, Ext.Direct is just a method
 attribute; it doesn't matter if it's called from Perl code or through
 Ext.Direct. This approach, in particular, allows for multi-tiered testing:
-    - Server side methods can be tested without setting up
-      HTTP environment with the usual tools like Test::More
-    - Server side classes can be tested as a whole via Ext.Direct calls
-      using Perl client
-    - Major application components are tested with browser automation
-      tools like Selenium.
+
+=over 4 
+
+=item *
+
+Server side methods can be tested without setting up HTTP environment
+with the usual tools like Test::More
+
+=item *
+
+Server side classes can be tested as a whole via Ext.Direct calls
+using Perl client
+
+=item *
+
+Major application components are tested with browser automation tools
+like Selenium.
+
+=back
 
 For more information on Ext.Direct, see
 L<http://www.sencha.com/products/extjs/extdirect/>.
@@ -482,83 +495,124 @@ L<http://www.sencha.com/products/extjs/extdirect/>.
 =head2 Terminology
 
 Ext.Direct uses the following terms, followed by their descriptions:
- Configuration  - Description of server side calls exposed to
-                  client side. Includes information on Action
-                  and Method names, as well as argument
-                  number and/or names.
- 
- API            - JavaScript chunk that encodes Configuration.
-                  Usually generated by application server
-                  called by client once upon startup.
- 
- Router         - Server side component that receives remoting
-                  calls, dispatches requests, collects and
-                  returns call Results or Exceptions.
- 
- Action         - Namespace unit; collection of Methods. The
-                  nearest Perl analog is package, other 
-                  languages may call it a Class. Since the
-                  actual calling code is JavaScript, Action
-                  names should conform to JavaScript naming
-                  rules (i.e. no '::', use dots instead).
- 
- Method         - Subroutine exposed through Ext.Direct API
-                  to be called by client side. Method is
-                  fully qualified by Action and Method names
-                  using dot as delimiter: Action.Method.
- 
- Result         - Any data returned by Method upon successful or
-                  unsuccessful call completion. This includes
-                  application logic errors. 'Not authenticated'
-                  and alike events should be returned as Results,
-                  not Exceptions.
- 
- Exception      - Fatal error, or any other unrecoverable event
-                  in application code. Calls that produce
-                  Exception instead of Result are considered
-                  unsuccessful; Ext.Direct provides built in
-                  mechanism for managing Exceptions.
- 
-                  Exceptions are not used to indicate errors in
-                  application logic flow, only for catastrophic
-                  conditions. Nearest analog is status code 500
-                  for HTTP responses.
- 
-                  Examples of Exceptions are: request JSON is
-                  broken and can't be decoded, called Method
-                  dies because of internall error, Result
-                  cannot be encoded in JSON, etc.
- 
- Event          - An asynchronous notification that can be
-                  generated by server side and passed to
-                  client side, resulting in some reaction.
-                  Events are useful for status updates, progress
-                  indicators and other predictably occuring
-                  conditions and events.
- 
- Event Provider - Server side script that gets polled by
-                  client side every N seconds; default N
-                  is 3 but it can be changed in client side
-                  configuration.
+
+=over 4
+
+=item Configuration
+
+Description of server side calls exposed to client side. Includes
+information on Action and Method names, as well as argument number
+and/or names
+
+=item API
+
+JavaScript chunk that encodes Configuration. Usually generated
+by application server and retrieved by client once upon startup.
+Another option is to embed API declaration in client side application
+code.
+
+=item Router
+
+Server side component that receives remoting calls, dispatches requests,
+collects and returns call Results or Exceptions.
+
+=item Action
+
+Namespace unit; collection of Methods. The nearest Perl analog is package,
+other languages may call it a Class. Since the actual calling code is
+JavaScript, Action names should conform to JavaScript naming rules
+(i.e. no '::', use dots instead).
+
+=item Method
+
+Subroutine exposed through Ext.Direct API to be called by client side.
+Method is fully qualified by Action and Method names using dot as
+delimiter: Action.Method.
+
+=item Result
+
+Any data returned by Method upon successful or unsuccessful call completion.
+This includes application logic errors. 'Not authenticated' and alike events
+should be returned as Results, not Exceptions.
+
+=item Exception
+
+Fatal error, or any other unrecoverable event in application code. Calls
+that produce Exception instead of Result are considered unsuccessful;
+Ext.Direct provides built in mechanism for managing Exceptions.
+
+Exceptions are not used to indicate errors in application logic flow,
+only for catastrophic conditions. Nearest analog is status code 500
+for HTTP responses.
+
+Examples of Exceptions are: request JSON is broken and can't be decoded;
+called Method dies because of internall error; Result cannot be encoded
+in JSON, etc.
+
+=item Event
+
+An asynchronous notification that can be generated by server side and
+passed to client side, resulting in some reaction. Events are useful
+for status updates, progress indicators and other predictably occuring
+conditions and events.
+
+=item Event Provider
+
+Server side script that gets polled by client side every N seconds;
+default N is 3 but it can be changed in client side configuration.
+
+=back
 
 =head1 USING RPC::EXTDIRECT
 
 In order to export subroutine to ExtDirect interface, use C<ExtDirect(n, ...)>
-attribute in sub's declaration. Note that there can be no space between
-attribute name and opening parentheses. n is mandatory calling convention
-declaration; it may be one of the following options:
-    - Number of arguments to be passed as ordered list
-    - Names of arguments to be passed as hash
-    - formHandler: method will receive hash of fields and file
-      uploads
-    - pollHandler: method that provides Events when polled by client
+attribute in sub declaration. Note that there can be no space between
+attribute name and opening parentheses. In Perls older than 5.12, attribute
+declaration can't span multiple lines, i.e. the whole C<ExtDirect(n, ...)>
+should fit in one line.
+
+n is mandatory calling convention declaration; it may be one of the following
+options:
+
+=over 4
+
+=item *
+
+Number of arguments to be passed as ordered list
+
+=item *
+
+Names of arguments to be passed as hash
+
+=item *
+
+formHandler: method will receive hash of fields and file uploads
+
+=item *
+
+pollHandler: method that provides Events when polled by client
+
+=back
 
 Optional method attributes can be specified after calling convention
 declaration, in hash-like C<key =E<gt> value> form. Optional attributes
 are:
-    - before: code reference to use as "before" hook. See L</HOOKS>.
-    - instead: code reference to "instead" hook.
-    - after: code reference to "after" hook.
+
+=over 4
+
+=item *
+
+before: code reference to use as "before" hook. See L</HOOKS>
+
+=item *
+
+instead: code reference to "instead" hook
+
+=item *
+
+after: code reference to "after" hook.
+
+=back
 
 =head1 METHODS
 
@@ -567,57 +621,74 @@ RPC::ExtDirect does not impose strict architectural notation on server
 side code. There is no mandatory object instantiation and no assumption
 about the code called. That said, an RPC::ExtDirect Method should conform
 to the following conventions:
-    - Be a class method, i.e. be aware that its first argument
-      will be package name. Just ignore it if you don't want it.
- 
-    - Ordered (numbered) arguments are passed as list in @_, so
-      $_[1] is the first argument. No more than number of arguments
-      declared in ExtDirect attribute will be passed to Method; any
-      extra will be dropped silently. Less actual arguments than
-      declared will result in Exception returned to client side,
-      and Method never gets called.
- 
-      The last argument is an environment object (see below). For
-      methods that take 0 arguments, it will be the first argument
-      after class name.
- 
-    - Named arguments are passed as hash in @_. No arguments other
-      than declared will be passed to Method; extra arguments will
-      be dropped silently. If not all arguments are present in
-      actual call, an Exception will be returned and Method never
-      gets called.
- 
-      Environment object will be passed in '_env' key.
- 
-    - Form handlers are passed their arguments as hash in @_.
-      Standard Ext.Direct form fields are removed from argument
-      hash; uploaded file(s) will be passed in file_uploads hash
-      element. It will only be present when there are uploaded
-      files. For more info, see "UPLOADS".
- 
-      Environment object will be passed in '_env' key.
- 
-    - All remoting Methods are called in scalar context. Returning
-      one scalar value is OK; returning array- or hashref is OK too.
-      Do not return blessed objects; it is almost always not
-      obvious how to serialize them into JSON that is expected by
-      client side; JSON encoder will choke and an Exception will
-      be returned to the client.
- 
-    - If an error is encountered while processing request, throw
-      an exception: die "My error string\n". Note that "\n" at
-      the end of error string; if you don't add it, die() will
-      append file name and line number to the error message;
-      which is probably not the best idea for errors that are not
-      shown in console but rather passed on to JavaScript client.
- 
-      RPC::ExtDirect will trim that last "\n" for you before
-      sending Exception back to client side.
- 
-    - Poll handler methods are called in list context and do not
-      receive any arguments except environment object. Return values
-      must be instantiated Event object(s), see RPC::ExtDirect::Event
-      for more detail.
+
+=over 4
+
+=item *
+
+Be a class method, i.e. be aware that its first argument will be package name.
+Just ignore it if you don't want it.
+
+=item *
+
+Ordered (numbered) arguments are passed as list in @_, so $_[1] is the first
+argument. No more than number of arguments declared in ExtDirect attribute
+will be passed to Method; any extra will be dropped silently. Less actual
+arguments than declared will result in Exception returned to client side,
+and Method never gets called.
+
+The last argument is an environment object (see L<ENVIRONMENT OBJECTS>).
+For methods that take 0 arguments, it will be the first argument after
+class name.
+
+=item *
+
+Named arguments are passed as hash in @_. No arguments other than declared
+will be passed to Method; extra arguments will be dropped silently. If not
+all arguments are present in actual call, an Exception will be returned and
+Method never gets called.
+
+Environment object will be passed in '_env' key.
+
+=item *
+
+Form handlers are passed their arguments as hash in @_. Standard Ext.Direct
+form fields are removed from argument hash; uploaded file(s) will be passed
+in file_uploads hash element. It will only be present when there are uploaded
+files. For more info, see L</UPLOADS>.
+
+Environment object will be passed in '_env' key.
+
+=item *
+
+All remoting Methods are called in scalar context. Returning one scalar value
+is OK; returning array- or hashref is OK too.
+
+Do not return blessed objects; it is almost always not obvious how to
+serialize them into JSON that is expected by
+client side; JSON encoder will choke and an Exception will
+be returned to the client.
+
+=item *
+
+If an error is encountered while processing request, throw
+an exception: die "My error string\n". Note that "\n" at
+the end of error string; if you don't add it, die() will
+append file name and line number to the error message;
+which is probably not the best idea for errors that are not
+shown in console but rather passed on to JavaScript client.
+
+RPC::ExtDirect will trim that last "\n" for you before
+sending Exception back to client side.
+
+=item *
+
+Poll handler methods are called in list context and do not
+receive any arguments except environment object. Return values
+must be instantiated Event object(s), see L<RPC::ExtDirect::Event>
+for more detail.
+
+=back
 
 =head1 HOOKS
 
@@ -634,31 +705,43 @@ user authorization, logging, cleanup, testing, etc.
 
 A hook is a Perl subroutine (can be anonymous, too). Hooks can be of three
 types:
-    - "Before" hook is called before the Method, and can be used
-      to change Method arguments or cancel Method execution. This
-      hook must return numeric value 1 to allow Method call. Any
-      other value will be interpreted as Ext.Direct Result; it
-      will be returned to client side and Method never gets called.
- 
-      Note that RPC::ExtDirect will not make any assumptions about
-      this hook's return value; returning a false value like '' or 0
-      will probably look not too helpful from client side code.
- 
-      If this hook throws an exception, it is returned as Ext.Direct
-      Exception to the client side, and the Method does not execute.
-  
-    - "Instead" hook replaces the Method it is assigned to. It is
-      the hook sub's responsibility to call (or not call) the Method
-      and return appropriate Result.
- 
-      If this hook throws an exception, it is interpreted as if the
-      Method trew it.
-  
-    - "After" hook is called after the Method or "instead" hook. This
-      hook cannot affect Method execution, it is intended mostly for
-      logging and testing purposes; its input include Method's
-      Result or Exception. This hook's return value and
-      thrown exceptions are ignored.
+
+=over 4
+
+=item *
+
+"Before" hook is called before the Method, and can be used
+to change Method arguments or cancel Method execution. This
+hook must return numeric value 1 to allow Method call. Any
+other value will be interpreted as Ext.Direct Result; it
+will be returned to client side and Method never gets called.
+
+Note that RPC::ExtDirect will not make any assumptions about
+this hook's return value; returning a false value like '' or 0
+will probably look not too helpful from client side code.
+
+If this hook throws an exception, it is returned as Ext.Direct
+Exception to the client side, and the Method does not execute.
+
+=item *
+
+"Instead" hook replaces the Method it is assigned to. It is
+the hook sub's responsibility to call (or not call) the Method
+and return appropriate Result.
+
+If this hook throws an exception, it is interpreted as if the
+Method trew it.
+
+=item *
+
+"After" hook is called after the Method or "instead" hook. This
+hook cannot affect Method execution, it is intended mostly for
+logging and testing purposes; its input include Method's
+Result or Exception.
+
+This hook's return value and thrown exceptions are ignored.
+
+=back
 
 Hooks can be defined on three levels, in order of precedence: method,
 package and global. For each Method, only one hook of each type can be
@@ -670,68 +753,110 @@ this way you can specify global and/or package hooks and exclude some
 specific Methods piecemeal.
 
 Hooks are subject to the following calling conventions:
-    - Hook subroutine is called as class method, i.e. first argument
-      is name of the package in which this sub was defined. Ignore it
-      if you don't need it.
+
+=over 4
+
+=item *
+
+Hook subroutine is called as class method, i.e. first argument
+is name of the package in which this sub was defined. Ignore it
+if you don't need it.
+
+=item *
+
+Hooks receive a hash of the following arguments:
+
+=over 8
+
+=item action
+
+Ext.Direct Action name for the Method
+
+=item method
+
+Ext.Direct Method name
+
+=item package
+
+Name of the package (not Action) where the Method is declared
+
+=item code
+
+Coderef to the Method subroutine
+
+=item param_no
  
-    - Hooks receive a hash of the following arguments:
-        action        => Ext.Direct Action name for the Method
- 
-        method        => Ext.Direct Method name
- 
-        package       => Name of the package (not Action) where
-                         the Method is declared
- 
-        code          => Coderef to the Method subroutine
- 
-        param_no      => Number of parameters when Method accepts
-                         ordered arguments
- 
-        param_names   => Arrayref with names of parameters when Method
-                         accepts named arguments
- 
-        formHandler   => True if Method handles form submits
- 
-        pollHandler   => True if Method handles Event poll requests
- 
-        arg           => Arrayref with actual arguments when Method
-                         accepts ordered args, single Environment
-                         object for poll handlers, hashref otherwise.
- 
-                         Note that this is a direct link to Method's @_
-                         so it is possible to modify the arguments
-                         in "before" hook
- 
-        env           => Environment object, see below. Like arg,
-                         this is direct reference to the same object
-                         that will be passed to Method, so it's
-                         possible to modify it in "before" hook
- 
-        before        => Coderef to "before" hook for that Method,
-                         or undef
- 
-        instead       => Coderef to "instead" hook for that Method,
-                         or undef
- 
-        after         => Coderef to "after" hook for that Method,
-                         or undef
- 
-        result        => For "after" hooks, the Result returned by
-                         Method or "instead" hook, if any. Does not
-                         exist for "before" and "instead" hooks
- 
-        exception     => For "after" hooks, an exception ($@) thrown
-                         by Method or "instead" hook, if any. Does
-                         not exist for "before" and "instead" hooks
- 
-        method_called => For "after" hooks, the reference to actual
-                         code called as Method, if any. Can be either
-                         Method itself, "instead" hook or undef if
-                         the call was canceled.
- 
-        orig          => A closure that binds Method coderef to
-                         its current arguments, allowing to call it
-                         as easily as $params{orig}->()
+Number of parameters when Method accepts ordered arguments
+
+=item param_names
+
+Arrayref with names of parameters when Method accepts named arguments
+
+=item formHandler
+
+True if Method handles form submits
+
+=item pollHandler
+
+True if Method handles Event poll requests
+
+=item arg
+
+Arrayref with actual arguments when Method
+accepts ordered args, single Environment
+object for poll handlers, hashref otherwise.
+
+Note that this is a direct link to Method's @_
+so it is possible to modify the arguments
+in "before" hook
+
+=item env
+
+Environment object, see below. Like arg,
+this is direct reference to the same object
+that will be passed to Method, so it's
+possible to modify it in "before" hook
+
+=item before
+
+Coderef to "before" hook for that Method, or undef
+
+=item instead
+
+Coderef to "instead" hook for that Method, or undef
+
+=item after
+
+Coderef to "after" hook for that Method, or undef
+
+=item result
+
+For "after" hooks, the Result returned by
+Method or "instead" hook, if any. Does not
+exist for "before" and "instead" hooks
+
+=item exception
+
+For "after" hooks, an exception ($@) thrown
+by Method or "instead" hook, if any. Does
+not exist for "before" and "instead" hooks
+
+=item method_called
+
+For "after" hooks, the reference to actual
+code called as Method, if any. Can be either
+Method itself, "instead" hook or undef if
+the call was canceled.
+
+=item orig
+
+A closure that binds Method coderef to
+its current arguments, allowing to call it
+as easily as $params{orig}->()
+
+=back
+
+=back
 
 =head1 ENVIRONMENT OBJECTS
 
@@ -747,12 +872,34 @@ across all platforms supported by RPC::ExtDirect, avoiding portability
 issues.
 
 The interface is modeled after de facto standard CGI.pm:
-    - C<$value = $env-E<gt>param('name')> will retrieve parameter by name
-    - C<@list = $env-E<gt>param()> will get the list of available parameters
-    - C<$cookie = $env-E<gt>cookie('name')> will retrieve a cookie
-    - C<@cookies = $env-E<gt>cookie()> will return the list of cookies
-    - C<$header = $env-E<gt>http('name')> will return HTTP header
-    - C<@headers = $env-E<gt>http()> will return the list of HTTP headers
+
+=over 4
+
+=item *
+
+C<$value = $env-E<gt>param('name')> will retrieve parameter by name
+
+=item *
+
+C<@list = $env-E<gt>param()> will get the list of available parameters
+
+=item *
+
+C<$cookie = $env-E<gt>cookie('name')> will retrieve a cookie
+
+=item *
+
+C<@cookies = $env-E<gt>cookie()> will return the list of cookies
+
+=item *
+
+C<$header = $env-E<gt>http('name')> will return HTTP header
+
+=item *
+
+C<@headers = $env-E<gt>http()> will return the list of HTTP headers
+
+=back
 
 Of course it is possible to use environment object in a more sophisticated
 way if you like to, however do not rely on it having a well-known class
@@ -770,12 +917,34 @@ In a formHandler Method, arguments are passed as a hash. If one or more
 file uploads were associated with request, the argument hash will contain
 'file_uploads' key with value set to arrayref of file hashrefs. Each file
 hashref will have the following keys:
-    - type     => MIME type of the file
-    - size     => file size, in octets
-    - path     => path to temporary file that holds uploaded content
-    - handle   => opened IO::Handle for temporary file
-    - basename => name portion of original file name
-    - filename => full original path as sent by client
+
+=over 4
+
+=item type
+
+MIME type of the file
+
+=item size
+
+file size, in octets
+
+=item path
+
+path to temporary file that holds uploaded content
+
+=item handle
+
+opened IO::Handle for temporary file
+
+=item basename
+
+name portion of original file name
+
+=item filename
+
+full original path as sent by client
+
+=back
 
 All files passed to a Method need to be processed in that Method; existence
 of temporary files is not guaranteed after Method returns.
@@ -822,12 +991,12 @@ any given Action.
 =head1 DEPENDENCIES
 
 RPC::ExtDirect is dependent on the following modules: L<Attribute::Handlers>,
-L<JSON>.
+L<"JSON">.
 
 =head1 BUGS AND LIMITATIONS
 
 In version 2.0, ExtDirect attribute was moved to BEGIN phase instead of
-default CHECK phase. While this improves compatibility with Plack and
+default CHECK phase. While this improves compatibility with
 Apache/mod_perl environments, this also causes backwards compatibility
 problems with Perl older than 5.12. Please let me know if you need to
 run RPC::ExtDirect 2.0 with older Perls; meanwhile RPC::ExtDirect 1.x
@@ -863,5 +1032,7 @@ on version 2.0 of RPC::ExtDirect suite of modules.
 Copyright (c) 2011-2012 by Alexander Tokarev.
 
 This module is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself. See L<perlartistic>.
+the same terms as Perl itself. See L<"perlartistic">.
+
+=cut
 
